//kazuya 3 freeze beta 5

#include <stdio.h>
#include <windows.h>
#include <string.h>
#include ".\mdk\machineinterface.h"
#include ".\mdk\mdk.h"
#pragma optimize ("awy", on)

CMachineParameter const paraNTick = 
{
	pt_word,
	"Length",  
	"Length",  
	1,                   
	0xFFFE,                  
	0xFFFF,
	MPF_STATE,
	16,                           
};

CMachineParameter const paraFreeze = 
{
	pt_byte,
	"Freeze",  
	"Freeze",  
	0,                   
	1,                  
	0xFF,
	MPF_STATE,
	0,                           
};

CMachineParameter const *pParameters[] =
{
	// global
	&paraNTick,
	&paraFreeze,
};

CMachineAttribute const attrPopUp = 
{
	"Popup(1=On)",
	0,
	1,
	1
};

CMachineAttribute const *pAttributes[] =
{
	&attrPopUp,
};

#pragma pack(1)

class gvals
{
public:
	word NTick;
	byte Freeze;
};

class avals
{
public:
	int PopUp;
};

#pragma pack()

CMachineInfo const MacInfo =
{
	MT_EFFECT,								// type
	MI_VERSION,
	MIF_DOES_INPUT_MIXING,					// flags
	0,										// min tracks
	0,										// max tracks
	2,										// numGlobalParameters
	0,										// numTrackParameters
	pParameters,
	1,
	pAttributes,
#ifdef _DEBUG
	"3 Freeze (Debug build)",				// name
#else
	"kazuya 3 Freeze",						// name
#endif
	"3 Freeze",								// short name
	"kazuya",								// author
	"Release\nAbout...",
};

class mi;

class miex : public CMDKMachineInterfaceEx
{
};

class mi : public CMDKMachineInterface
{
public:
	mi();
	virtual ~mi();

	virtual void Tick();
	virtual void Stop();
	virtual void MDKInit(CMachineDataInput * const pi);
	virtual void MDKSave(CMachineDataOutput * const po);
	virtual bool MDKWork(float *psamples, int numsamples, int const mode);
	virtual bool MDKWorkStereo(float *psamples, int numsamples, int const mode);
	virtual void Command(int const i);
	virtual char const *mi::DescribeValue(int const param, int const value);
	virtual CMDKMachineInterfaceEx *GetEx(){return &ex;}
	virtual void OutputModeChanged(bool stereo){}
	virtual void AttributesChanged();

public:
	miex ex;

private:
	gvals gval;
	avals aval;
	bool Initialising;
	int popup;

	word ntick;
	word tick_count;
	int freeze;
	char reg_file[256];
	char freeze_file[256];
	int temp_file;
	bool frozen;
	FILE *fp;

	CMachine *ThisMac;
};

DLL_EXPORTS

mi::mi()
{
	GlobalVals = &gval;
	AttrVals = (int*)&aval;
}

mi::~mi()
{
}

void mi::MDKInit(CMachineDataInput * const pi)
{
	Initialising=true;
	SetOutputMode(true);
	ThisMac=pCB->GetThisMachine();
	tick_count=0;
	ntick=16;
	freeze=0;
	fp=NULL;

	frozen=false;
	temp_file=-1;
	if(pi)
	{
		pi->Read(frozen);
		pi->Read(temp_file);
	}

	HKEY hk;
	if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Jeskola\\Buzz\\Settings",0,KEY_QUERY_VALUE,&hk)==ERROR_SUCCESS)
	{		
		DWORD dwType;
		DWORD dwSize=256;					
		if(RegQueryValueEx(hk,"BuzzPath",NULL,&dwType,(BYTE *)reg_file,&dwSize)==ERROR_SUCCESS)
		{							
			int len=strlen(reg_file);
			if(reg_file[len-1]!='\\') 
			{
				reg_file[len]='\\';
				reg_file[len+1]=0;
			}
			strcat(reg_file,"Gear\\Effects\\kazuya 3 freeze.fz");
		}
		RegCloseKey(hk);
	}

	if(temp_file!=-1)
		sprintf(freeze_file,"%s%d",reg_file,temp_file);
}

void mi::MDKSave(CMachineDataOutput * const po)
{
	po->Write(frozen);
	po->Write(temp_file);
}

void mi::AttributesChanged()
{
	popup=aval.PopUp;
}

void mi::Tick()
{
	if(gval.NTick != paraNTick.NoValue)
		ntick=gval.NTick;
	
	if(!Initialising)
	{
		if(fp!=NULL && tick_count>=ntick)
		{
			fclose(fp);
			if(freeze && !frozen)
			{
				frozen=true;
				if(popup)
					pCB->MessageBox("Please mute frozen inputs.");
			}
		}

		if(gval.Freeze != paraFreeze.NoValue)
		{
			tick_count=0;
			freeze=gval.Freeze;
			if(fp!=NULL)
				fclose(fp);
			if(!freeze)
				frozen=false;
			else
			{
				if(!frozen)
				{
					if(temp_file==-1)
					{
						char temp_freeze_file[256]={0};
						
						while(1)
						{
							temp_file++;
							sprintf(temp_freeze_file,"%s%d",reg_file,temp_file);
							fp=fopen(temp_freeze_file,"rb");
							if(fp==NULL)
							{
								strncpy(freeze_file,temp_freeze_file,256);
								break;
							}
							fclose(fp);
						}
					}
					fp=fopen(freeze_file,"wb");
				}
				else
				{
					fp=fopen(freeze_file,"rb");
					if(fp==NULL)
					{
						temp_file=-1;
						pCB->ControlChange(ThisMac,1,0,1,0);
					}
				}
			}
		}
	
		if(freeze && tick_count<=ntick)
			tick_count++;
	}
}

void mi::Stop()
{
	tick_count=0;
	if(fp!=NULL)
	{
		fclose(fp);
		if(freeze && !frozen)
		{
			frozen=true;
			if(popup)
				pCB->MessageBox("Please mute frozen inputs.");
		}
	}
}

bool mi::MDKWork(float *psamples, int numsamples, int const mode)
{
	return false;
}

bool mi::MDKWorkStereo(float *psamples, int numsamples, int const mode)
{
	Initialising=false;
	
	if(mode==WM_NOIO || tick_count>ntick)
		return false;

	if(mode==WM_READ)
		return true;

	if(fp==NULL)
	{
		if(mode==WM_WRITE)
			return false;
		else
			return true;
	}

	if(mode==WM_WRITE)
	{
		if(!frozen)
		{
			float *zero=new float[MAX_BUFFER_LENGTH*2];
			memset(zero,0,MAX_BUFFER_LENGTH*2*sizeof(float));
			fwrite(zero,sizeof(float),numsamples*2,fp);
			delete[] zero;
			return false;
		}
	}

	if(!frozen)
		fwrite(psamples,sizeof(float),numsamples*2,fp);
	else
	{
		if(fread(psamples,sizeof(float),numsamples*2,fp)!=(unsigned int)numsamples*2)
			return false;
	}
	return true;
}

void mi::Command(int const i)
{
	switch(i)
	{
	case 0:
		Stop();
		temp_file=-1;
		pCB->ControlChange(ThisMac,1,0,1,0);
		remove(freeze_file);
		break;
	case 1:
		pCB->MessageBox("kazuya 3 Freeze\n2005\n\nhttp://www33.ocn.ne.jp/~brandcafeclub/\nyazmad@hotmail.com");
		break;
	default:
		break;
	}
}

char const *mi::DescribeValue(int const param, int const value)
{
	static char txt[16];

	switch(param)
	{
	case 0:
		sprintf(txt,"%d ticks",value);
		return txt;
		break;
	case 1:
		if(!value)
			sprintf(txt,"Off");
		else
		{
			if(!frozen)
				sprintf(txt,"Recording...");
			else
				sprintf(txt,"PlayBack");
		}
		return txt;
		break;
	default: return NULL;
		break;
	};
}

#pragma optimize ("", on)